==> par-tasker-4.0.2-sources.jar: com/xebialabs/deployit/engine/tasker/TaskExecutionContext.scala <==
package com.xebialabs.deployit.engine.tasker

import com.xebialabs.deployit.plugin.api.flow.ExecutionContext
import com.xebialabs.deployit.plugin.api.services.Repository
import com.xebialabs.deployit.plugin.api.inspection.InspectionContext
import com.xebialabs.deployit.engine.spi.execution.ExecutionStateListener
import java.io.{PrintWriter, StringWriter, Serializable}
import grizzled.slf4j.Logger
import com.xebialabs.deployit.plugin.api.execution.ExecutionContextListener


class TaskExecutionContext(parent: Option[TaskExecutionContext], spec: TaskSpecification) extends Serializable {
  private[this] val attributes: Cache[String, AnyRef] = Cache()
  @transient var repository: Repository = null

  if (spec != null) {
    import collection.convert.wrapAll._
    spec.getListeners.foreach(l => attributes.put(l.getClass.getName, l))
  }

  private def getParent: Option[TaskExecutionContext] = parent

  val childContexts: Cache[BlockPath, TaskExecutionContext] = Cache()

  def contextFor(block: Block): TaskExecutionContext = contextFor(block.id)

  def contextFor(path: BlockPath) : TaskExecutionContext = childContexts.get(path).getOrElse({
    val c = new TaskExecutionContext(Some(this), null)
    childContexts.put(path, c)
    c
  })

  private[tasker] def allListeners: Set[ExecutionStateListener] = {
    def getRootContext(tec: TaskExecutionContext): TaskExecutionContext = tec.getParent match {
      case Some(pp) => getRootContext(pp)
      case None => tec
    }

    def collectListeners(t: TaskExecutionContext): Set[ExecutionStateListener] = t.getAttributes.collect({
      case (n, l) if l.isInstanceOf[ExecutionStateListener] => l.asInstanceOf[ExecutionStateListener]
    }).toSet
    def recursiveCollectListeners(t: TaskExecutionContext): Set[ExecutionStateListener] = collectListeners(t) ++ t.childContexts.values.flatMap(recursiveCollectListeners)
    recursiveCollectListeners(getRootContext(this))
  }

  private[tasker] def listeners: Set[ExecutionStateListener] = {
    val collect: Set[ExecutionStateListener] = attributes.collect({
      case (n, l) if l.isInstanceOf[ExecutionStateListener] => l.asInstanceOf[ExecutionStateListener]
    }).toSet
    parent match {
      case None => collect
      case Some(p) => collect ++ p.listeners
    }
  }

  def clearChildContexts() = childContexts.clear()

  def getAttribute(name: String): AnyRef = attributes.get(name).getOrElse(parent match {
    case None => null
    case Some(p) => p.getAttribute(name)
  }) match {
    case l : OldExecutionContextListenerCleanupTrigger => l.getWrappedListener
    case l @ _                                         => l
  }

  def setAttribute(name: String, v: AnyRef) = v match {
    case ecl: ExecutionContextListener => attributes.put(name, new OldExecutionContextListenerCleanupTrigger(name, ecl))
    case x @ _ => attributes.put(name, x)
  }

  private[tasker] def getAttributes = attributes

  def unsetAttribute(name: String) = attributes.remove(name)

  def getRepository = repository

  def stepContext(step: TaskStep) = new ExecutionContext {
    val ERROR_PREFIX = "[ERROR]: "

    val stepLogger = Logger.apply(step.getImplementation.getClass)

    def setAttribute(name: String, value: scala.AnyRef) {
      TaskExecutionContext.this.setAttribute(name, value)
    }

    def logOutput(output: String) {
      stepLogger.info(output)
      step.logBuilder.append(output).append("\n")
      step.touch()
    }

    def logError(error: String) {
      stepLogger.error(error)
      appendErrorToLog(error)
      step.touch()
    }

    def logError(error: String, t: Throwable) {
      stepLogger.error(error, t)
      appendErrorToLog(error)
      val stringWriter: StringWriter = new StringWriter
      t.printStackTrace(new PrintWriter(stringWriter))
      for (stackTraceLine <- stringWriter.toString.split("\n")) {
        appendErrorToLog(stackTraceLine)
      }
      step.touch()
    }

    def appendErrorToLog(error: String) {
      step.logBuilder.append(ERROR_PREFIX).append(error).append("\n")
    }

    def getAttribute(name: String): AnyRef = attributes.getOrElse(name, null) match {
      case l : OldExecutionContextListenerCleanupTrigger => l.getWrappedListener
      case l @ _                                         => l
    }

    def getInspectionContext: InspectionContext = spec.getInspectionContext

    def getRepository: Repository = repository
  }
}

==> par-tasker-2014.3-sources.jar: com/xebialabs/deployit/engine/tasker/TaskExecutionContext.scala <==
package com.xebialabs.deployit.engine.tasker

import com.xebialabs.deployit.plugin.api.flow.{ITask, ExecutionContext}
import com.xebialabs.deployit.plugin.api.services.Repository
import com.xebialabs.deployit.plugin.api.inspection.InspectionContext
import com.xebialabs.deployit.engine.spi.execution.ExecutionStateListener
import java.io.{PrintWriter, StringWriter, Serializable}
import grizzled.slf4j.Logger
import com.xebialabs.deployit.plugin.api.execution.ExecutionContextListener
import java.util


class TaskExecutionContext(parent: Option[TaskExecutionContext], spec: TaskSpecification) extends Serializable {
  private[this] val attributes: Cache[String, AnyRef] = Cache()
  @transient var repository: Repository = null

  if (spec != null) {
    import collection.convert.wrapAll._
    spec.getListeners.foreach(l => attributes.put(l.getClass.getName, l))
  }

  private def getParent: Option[TaskExecutionContext] = parent

  val childContexts: Cache[BlockPath, TaskExecutionContext] = Cache()

  def contextFor(block: Block): TaskExecutionContext = contextFor(block.id)

  def contextFor(path: BlockPath): TaskExecutionContext = childContexts.get(path).getOrElse({
    val c = new TaskExecutionContext(Some(this), null)
    childContexts.put(path, c)
    c
  })

  private[tasker] def allListeners: Set[ExecutionStateListener] = {
    def getRootContext(tec: TaskExecutionContext): TaskExecutionContext = tec.getParent match {
      case Some(pp) => getRootContext(pp)
      case None => tec
    }

    def collectListeners(t: TaskExecutionContext): Set[ExecutionStateListener] = t.getAttributes.collect({
      case (n, l) if l.isInstanceOf[ExecutionStateListener] => l.asInstanceOf[ExecutionStateListener]
    }).toSet
    def recursiveCollectListeners(t: TaskExecutionContext): Set[ExecutionStateListener] = collectListeners(t) ++ t.childContexts.values.flatMap(recursiveCollectListeners)
    recursiveCollectListeners(getRootContext(this))
  }

  private[tasker] def listeners: Set[ExecutionStateListener] = {
    val collect: Set[ExecutionStateListener] = attributes.collect({
      case (n, l) if l.isInstanceOf[ExecutionStateListener] => l.asInstanceOf[ExecutionStateListener]
    }).toSet
    parent match {
      case None => collect
      case Some(p) => collect ++ p.listeners
    }
  }

  def clearChildContexts() = childContexts.clear()

  def getAttribute(name: String): AnyRef = attributes.get(name).getOrElse(parent match {
    case None => null
    case Some(p) => p.getAttribute(name)
  }) match {
    case l: OldExecutionContextListenerCleanupTrigger => l.getWrappedListener
    case l@_ => l
  }

  def setAttribute(name: String, v: AnyRef) = v match {
    case ecl: ExecutionContextListener => attributes.put(name, new OldExecutionContextListenerCleanupTrigger(name, ecl))
    case x@_ => attributes.put(name, x)
  }

  private[tasker] def getAttributes = attributes

  def unsetAttribute(name: String) = attributes.remove(name)

  def getRepository = repository

  def stepContext(step: TaskStep, task: Task) = new ExecutionContext {
    val ERROR_PREFIX = "[ERROR]: "

    val stepLogger = Logger.apply(step.getImplementation.getClass)

    def setAttribute(name: String, value: scala.AnyRef) {
      TaskExecutionContext.this.setAttribute(name, value)
    }

    def logOutput(output: String) {
      stepLogger.info(output)
      step.logBuilder.append(output).append("\n")
      step.touch()
    }

    def logError(error: String) {
      stepLogger.error(error)
      appendErrorToLog(error)
      step.touch()
    }

    def logError(error: String, t: Throwable) {
      stepLogger.error(error, t)
      appendErrorToLog(error)
      val stringWriter: StringWriter = new StringWriter
      t.printStackTrace(new PrintWriter(stringWriter))
      for (stackTraceLine <- stringWriter.toString.split("\n")) {
        appendErrorToLog(stackTraceLine)
      }
      step.touch()
    }

    def appendErrorToLog(error: String) {
      step.logBuilder.append(ERROR_PREFIX).append(error).append("\n")
    }

    def getAttribute(name: String): AnyRef = attributes.getOrElse(name, null) match {
      case l: OldExecutionContextListenerCleanupTrigger => l.getWrappedListener
      case l@_ => l
    }

    def getInspectionContext: InspectionContext = spec.getInspectionContext

    def getRepository: Repository = repository

    def getTask: ITask = new ITask {

      override def getMetadata: util.Map[String, String] = task.getMetadata

      override def getUsername: String = task.getOwner

      override def getId: String = task.getId
    }

  }
}
